Package: modelr
Title: Modelling Functions that Work with the Pipe
Version: 0.1.11
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Functions for modelling that help you seamlessly integrate
    modelling into a pipeline of data manipulation and visualisation.
License: GPL-3
URL: https://modelr.tidyverse.org, https://github.com/tidyverse/modelr
BugReports: https://github.com/tidyverse/modelr/issues
Depends: R (>= 3.2)
Imports: broom, magrittr, purrr (>= 0.2.2), rlang (>= 1.0.6), tibble,
        tidyr (>= 0.8.0), tidyselect, vctrs
Suggests: compiler, covr, ggplot2, testthat (>= 3.0.0)
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2023-03-21 14:12:54 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2023-03-22 13:00:07 UTC
Built: R 4.6.0; ; 2026-04-25 13:55:06 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,permutation)
S3method(as.data.frame,resample)
S3method(as.integer,permutation)
S3method(as.integer,resample)
S3method(dim,permutation)
S3method(dim,resample)
S3method(obj_sum,resample)
S3method(predictor_vars,default)
S3method(print,permutation)
S3method(print,resample)
S3method(response_var,default)
S3method(typical,character)
S3method(typical,factor)
S3method(typical,integer)
S3method(typical,logical)
S3method(typical,numeric)
S3method(typical,ordered)
export("%>%")
export(add_predictions)
export(add_predictors)
export(add_residuals)
export(bootstrap)
export(crossv_kfold)
export(crossv_loo)
export(crossv_mc)
export(data_grid)
export(fit_with)
export(formulae)
export(formulas)
export(gather_predictions)
export(gather_residuals)
export(geom_ref_line)
export(mae)
export(mape)
export(model_matrix)
export(mse)
export(na.warn)
export(permute)
export(permute_)
export(qae)
export(resample)
export(resample_bootstrap)
export(resample_partition)
export(resample_permutation)
export(rmse)
export(rsae)
export(rsquare)
export(seq_range)
export(spread_predictions)
export(spread_residuals)
export(typical)
import(rlang)
importFrom(broom,tidy)
importFrom(magrittr,"%>%")
importFrom(purrr,compact)
importFrom(purrr,map)
importFrom(purrr,reduce)
importFrom(stats,quantile)
importFrom(tibble,obj_sum)

# modelr <img src="man/figures/logo.png" align="right" />

<!-- badges: start -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R build
status](https://github.com/tidyverse/modelr/workflows/R-CMD-check/badge.svg)](https://github.com/tidyverse/modelr/actions)
[![Codecov test
coverage](https://codecov.io/gh/tidyverse/modelr/branch/main/graph/badge.svg)](https://app.codecov.io/gh/tidyverse/modelr?branch=main)
<!-- badges: end -->

## Overview

The modelr package provides functions that help you create elegant
pipelines when modelling. It is designed primarily to support teaching
the basics of modelling within the tidyverse, particularly in [R for
Data Science](https://r4ds.had.co.nz/model-basics.html).

Please see <https://www.tidymodels.org/> for a more comprehensive
framework for modelling within the tidyverse.

## Installation

``` r
# The easiest way to get modelr is to install the whole tidyverse:
install.packages("tidyverse")

# Alternatively, install just modelr:
install.packages("modelr")

# Or the development version from GitHub:
# install.packages("devtools")
devtools::install_github("tidyverse/modelr")
```

## Status

modelr is stable: it has achieved its goal of making it easier to teach
modelling within the tidyverse. For more general modelling tasks, check
out the family of “tidymodel” packages like
[recipes](https://topepo.github.io/recipes/),
[rsample](https://topepo.github.io/rsample/),
[parsnip](https://topepo.github.io/parsnip/), and
[tidyposterior](https://topepo.github.io/tidyposterior/).

## Getting started

``` r
library(modelr)
```

### Partitioning and sampling

The `resample` class stores a “reference” to the original dataset and a
vector of row indices. A resample can be turned into a dataframe by
calling `as.data.frame()`. The indices can be extracted using
`as.integer()`:

``` r
# a subsample of the first ten rows in the data frame
rs <- resample(mtcars, 1:10)
as.data.frame(rs)
#>                    mpg cyl  disp  hp drat    wt  qsec vs am gear carb
#> Mazda RX4         21.0   6 160.0 110 3.90 2.620 16.46  0  1    4    4
#> Mazda RX4 Wag     21.0   6 160.0 110 3.90 2.875 17.02  0  1    4    4
#> Datsun 710        22.8   4 108.0  93 3.85 2.320 18.61  1  1    4    1
#> Hornet 4 Drive    21.4   6 258.0 110 3.08 3.215 19.44  1  0    3    1
#> Hornet Sportabout 18.7   8 360.0 175 3.15 3.440 17.02  0  0    3    2
#> Valiant           18.1   6 225.0 105 2.76 3.460 20.22  1  0    3    1
#> Duster 360        14.3   8 360.0 245 3.21 3.570 15.84  0  0    3    4
#> Merc 240D         24.4   4 146.7  62 3.69 3.190 20.00  1  0    4    2
#> Merc 230          22.8   4 140.8  95 3.92 3.150 22.90  1  0    4    2
#> Merc 280          19.2   6 167.6 123 3.92 3.440 18.30  1  0    4    4
as.integer(rs)
#>  [1]  1  2  3  4  5  6  7  8  9 10
```

The class can be utilized in generating an exclusive partitioning of a
data frame:

``` r
# generate a 30% testing partition and a 70% training partition
ex <- resample_partition(mtcars, c(test = 0.3, train = 0.7))
lapply(ex, dim)
#> $test
#> [1]  9 11
#> 
#> $train
#> [1] 23 11
```

modelr offers several resampling methods that result in a list of
`resample` objects (organized in a data frame):

``` r
# bootstrap
boot <- bootstrap(mtcars, 100)
# k-fold cross-validation
cv1 <- crossv_kfold(mtcars, 5)
# Monte Carlo cross-validation
cv2 <- crossv_mc(mtcars, 100)

dim(boot$strap[[1]])
#> [1] 32 11
dim(cv1$train[[1]])
#> [1] 25 11
dim(cv1$test[[1]])
#> [1]  7 11
dim(cv2$train[[1]])
#> [1] 25 11
dim(cv2$test[[1]])
#> [1]  7 11
```

### Model quality metrics

modelr includes several often-used model quality metrics:

``` r
mod <- lm(mpg ~ wt, data = mtcars)
rmse(mod, mtcars)
#> [1] 2.949163
rsquare(mod, mtcars)
#> [1] 0.7528328
mae(mod, mtcars)
#> [1] 2.340642
qae(mod, mtcars)
#>        5%       25%       50%       75%       95% 
#> 0.1784985 1.0005640 2.0946199 3.2696108 6.1794815
```

### Interacting with models

A set of functions let you seamlessly add predictions and residuals as
additional columns to an existing data frame:

``` r
set.seed(1014)
df <- tibble::tibble(
  x = sort(runif(100)),
  y = 5 * x + 0.5 * x ^ 2 + 3 + rnorm(length(x))
)

mod <- lm(y ~ x, data = df)
df %>% add_predictions(mod)
#> # A tibble: 100 × 3
#>          x     y  pred
#>      <dbl> <dbl> <dbl>
#>  1 0.00740 3.90   3.08
#>  2 0.0201  2.86   3.15
#>  3 0.0280  2.93   3.19
#>  4 0.0281  3.16   3.19
#>  5 0.0312  3.19   3.21
#>  6 0.0342  3.72   3.23
#>  7 0.0514  0.984  3.32
#>  8 0.0586  5.98   3.36
#>  9 0.0637  2.96   3.39
#> 10 0.0652  3.54   3.40
#> # … with 90 more rows
df %>% add_residuals(mod)
#> # A tibble: 100 × 3
#>          x     y   resid
#>      <dbl> <dbl>   <dbl>
#>  1 0.00740 3.90   0.822 
#>  2 0.0201  2.86  -0.290 
#>  3 0.0280  2.93  -0.256 
#>  4 0.0281  3.16  -0.0312
#>  5 0.0312  3.19  -0.0223
#>  6 0.0342  3.72   0.496 
#>  7 0.0514  0.984 -2.34  
#>  8 0.0586  5.98   2.62  
#>  9 0.0637  2.96  -0.428 
#> 10 0.0652  3.54   0.146 
#> # … with 90 more rows
```

For visualization purposes it is often useful to use an evenly spaced
grid of points from the data:

``` r
data_grid(mtcars, wt = seq_range(wt, 10), cyl, vs)
#> # A tibble: 60 × 3
#>       wt   cyl    vs
#>    <dbl> <dbl> <dbl>
#>  1  1.51     4     0
#>  2  1.51     4     1
#>  3  1.51     6     0
#>  4  1.51     6     1
#>  5  1.51     8     0
#>  6  1.51     8     1
#>  7  1.95     4     0
#>  8  1.95     4     1
#>  9  1.95     6     0
#> 10  1.95     6     1
#> # … with 50 more rows

# For continuous variables, seq_range is useful
mtcars_mod <- lm(mpg ~ wt + cyl + vs, data = mtcars)
data_grid(mtcars, wt = seq_range(wt, 10), cyl, vs) %>% add_predictions(mtcars_mod)
#> # A tibble: 60 × 4
#>       wt   cyl    vs  pred
#>    <dbl> <dbl> <dbl> <dbl>
#>  1  1.51     4     0  28.4
#>  2  1.51     4     1  28.9
#>  3  1.51     6     0  25.6
#>  4  1.51     6     1  26.2
#>  5  1.51     8     0  22.9
#>  6  1.51     8     1  23.4
#>  7  1.95     4     0  27.0
#>  8  1.95     4     1  27.5
#>  9  1.95     6     0  24.2
#> 10  1.95     6     1  24.8
#> # … with 50 more rows
```

## Code of conduct

Please note that this project is released with a [Contributor Code of
Conduct](https://modelr.tidyverse.org/CODE_OF_CONDUCT.html). By
participating in this project you agree to abide by its terms.

#' Generate test-training pairs for cross-validation
#'
#' `crossv_kfold` splits the data into `k` exclusive partitions,
#' and uses each partition for a test-training split. `crossv_mc`
#' generates `n` random partitions, holding out `test` of the
#' data for training. `crossv_loo` performs leave-one-out
#' cross-validation, i.e., `n = nrow(data)` training partitions containing
#' `n - 1` rows each.
#'
#' @inheritParams resample_partition
#' @param n Number of test-training pairs to generate (an integer).
#' @param test Proportion of observations that should be held out for testing
#'   (a double).
#' @param id Name of variable that gives each model a unique integer id.
#' @return A data frame with columns `test`, `train`, and `.id`.
#'   `test` and `train` are list-columns containing [resample()] objects.
#'   The number of rows is `n` for `crossv_mc()`, `k` for `crossv_kfold()`
#'   and `nrow(data)` for `crossv_loo()`.
#' @export
#' @examples
#' cv1 <- crossv_kfold(mtcars, 5)
#' cv1
#'
#' library(purrr)
#' cv2 <- crossv_mc(mtcars, 100)
#' models <- map(cv2$train, ~ lm(mpg ~ wt, data = .))
#' errs <- map2_dbl(models, cv2$test, rmse)
#' hist(errs)
crossv_mc <- function(data, n, test = 0.2, id = ".id") {
  if (!is.numeric(n) || length(n) != 1) {
    stop("`n` must be a single integer.", call. = FALSE)
  }
  if (!is.numeric(test) || length(test) != 1 || test <= 0 || test >= 1) {
    stop("`test` must be a value between 0 and 1.", call. = FALSE)
  }

  p <- c(train = 1 - test, test = test)
  runs <- purrr::rerun(n, resample_partition(data, p))
  cols <- purrr::transpose(runs)
  cols[[id]] <- id(n)

  tibble::as_tibble(cols)
}

#' @export
#' @param k Number of folds (an integer).
#' @rdname crossv_mc
crossv_kfold <- function(data, k = 5, id = ".id") {
  if (!is.numeric(k) || length(k) != 1) {
    stop("`k` must be a single integer.", call. = FALSE)
  }

  n <- nrow(data)
  folds <- sample(rep(1:k, length.out = n))

  idx <- seq_len(n)
  fold_idx <- split(idx, folds)

  fold <- function(test) {
    list(
      train = resample(data, setdiff(idx, test)),
      test = resample(data, test)
    )
  }

  cols <- purrr::transpose(purrr::map(fold_idx, fold))
  cols[[id]] <- id(k)

  tibble::as_tibble(cols)
}

#' @export
#' @rdname crossv_mc
crossv_loo <- function(data, id = ".id") {

  n <- nrow(data)
  idx <- seq_len(n)
  fold_idx <- split(idx, idx)

  fold <- function(test) {
    list(
      train = resample(data, setdiff(idx, test)),
      test = resample(data, test)
    )
  }

  cols <- purrr::transpose(purrr::map(fold_idx, fold))
  cols[[id]] <- 1:n

  tibble::as_tibble(cols)
}

test_that("number of crossfolds is a single integer", {
  expect_error(crossv_kfold(mtcars, k = "0"), "single integer")
  expect_error(crossv_kfold(mtcars, k = c(1, 2)), "single integer")
})

test_that("% of observations held out for testing is between 0 and 1", {
  expect_error(crossv_mc(mtcars, n = 100, test = 2), "value between")
})

test_that("number of training pairs to generate is a single integer", {
  expect_error(crossv_mc(mtcars, n = c(1,2)), "single integer")
  expect_error(crossv_mc(mtcars, n = "0"), "single integer")
})

test_that("crossv_kfold() and crossv_mc() return tibbles", {
  out <- crossv_kfold(mtcars, 5)
  expect_s3_class(out, "tbl_df")

  out <- crossv_mc(mtcars, 5)
  expect_s3_class(out, "tbl_df")
})

d <- head(mtcars, 30)
d$row <- 1:30

test_that("Can perform cross validation", {
  cross <- crossv_kfold(d, k = 5)

  expect_equal(nrow(cross), 5)

  trains <- map(cross$train, as.data.frame)
  expect_true(all(purrr::map_dbl(trains, nrow) == 24))

  tests <- map(cross$test, as.data.frame)
  expect_true(all(purrr::map_dbl(tests, nrow) == 6))

  overlaps <- purrr::map2(purrr::map(trains, "row"), purrr::map(tests, "row"), intersect)
  expect_true(all(lengths(overlaps) == 0))
})

test_that("Can perform leave-one-out cross validation", {
  cross <- crossv_loo(d)

  expect_equal(nrow(cross), 30)

  trains <- map(cross$train, as.data.frame)
  expect_true(all(purrr::map_dbl(trains, nrow) == 29))

  tests <- map(cross$test, as.data.frame)
  expect_true(all(purrr::map_dbl(tests, nrow) == 1))

  overlaps <- purrr::map2(purrr::map(trains, "row"), purrr::map(tests, "row"), intersect)
  expect_true(all(lengths(overlaps) == 0))

  expect_true(cross[nrow(cross),]$.id==nrow(cross))
  expect_true(cross[1,]$.id==1)
})



test_that("Can perform bootstrapping", {
  boot <- mtcars %>%
    bootstrap(5)

  expect_equal(nrow(boot), 5)

  for (b in boot$strap) {
    bd <- as.data.frame(b)
    expect_equal(nrow(bd), nrow(mtcars))
    expect_true(all(bd$mpg %in% mtcars$mpg))
    expect_false(all(bd$mpg == mtcars$mpg))
  }
})


test_that("Can perform permutation", {
  perm <- mtcars %>%
    permute(5, mpg)

  expect_equal(nrow(perm), 5)

  for (p in perm$perm) {
    pd <- as.data.frame(p)
    expect_equal(mtcars$cyl, pd$cyl)
    expect_equal(mtcars$hp, pd$hp)
    expect_equal(sort(mtcars$mpg), sort(pd$mpg))
    # chance of permutation being the same is basically nil
    expect_false(all(mtcars$mpg == pd$mpg))
  }
})

test_that("appropriate args are supplied to resample_*()", {
  expect_error(resample("mtcars", 1:10), "data frame")
  expect_error(resample(mtcars, 10), "integer vector")
  expect_error(
    resample_permutation(mtcars,  c("mpg", "cyl"), idx = 2),
    "same length"
  )
  expect_error(
    resample_permutation("mtcars",  c("mpg", "cyl"), idx = 1:32),
    "data frame"
  )
  expect_error(
    resample_permutation(mtcars,  c("mpg", "iris"), idx = 1:32),
    "vector"
  )
})

#' Compute model quality for a given dataset
#'
#' @description
#' Three summaries are immediately interpretible on the scale of the response
#' variable:
#' * `rmse()` is the root-mean-squared-error
#' * `mae()` is the mean absolute error
#' * `qae()` is quantiles of absolute error.
#'
#' Other summaries have varying scales and interpretations:
#' * `mape()` mean absolute percentage error.
#' * `rsae()` is the relative sum of absolute errors.
#' * `mse()` is the mean-squared-error.
#' * `rsquare()` is the variance of the predictions divided by the
#'   variance of the response.
#' @param model A model
#' @param data The dataset
#' @name model-quality
#' @examples
#' mod <- lm(mpg ~ wt, data = mtcars)
#' mse(mod, mtcars)
#' rmse(mod, mtcars)
#' rsquare(mod, mtcars)
#' mae(mod, mtcars)
#' qae(mod, mtcars)
#' mape(mod, mtcars)
#' rsae(mod, mtcars)
NULL

#' @export
#' @rdname model-quality
mse <- function(model, data) {
  x <- residuals(model, data)
  mean(x^2, na.rm = TRUE)
}

#' @export
#' @rdname model-quality
rmse <- function(model, data) {
  x <- residuals(model, data)
  sqrt(mean(x^2, na.rm = TRUE))
}

#' @export
#' @rdname model-quality
mae <- function(model, data) {
  x <- residuals(model, data)
  mean(abs(x), na.rm = TRUE)
}

#' @export
#' @rdname model-quality
rsquare <- function(model, data) {
  x <- residuals(model, data)
  y <- response(model, data)

  1 - stats::var(x, na.rm = TRUE) / stats::var(y, na.rm = TRUE)
}

#' @export
#' @rdname model-quality
#' @param probs Numeric vector of probabilities
qae <- function(model, data, probs = c(0.05, 0.25, 0.5, 0.75, 0.95)) {
  x <- residuals(model, data)
  stats::quantile(abs(x), probs, na.rm = TRUE)
}

#' @export
#' @rdname model-quality
mape <- function(model, data) {
  x <- residuals(model, data)
  y <- response(model, data)
  mean(abs(x / y), na.rm = TRUE)
}

#' @export
#' @rdname model-quality
rsae <- function(model, data) {
  x <- residuals(model, data)
  y <- response(model, data)

  miss <- is.na(x) | is.na(y)
  sum(abs(x[!miss])) / sum(abs(y[!miss]))
}

#' Add a reference line (ggplot2).
#'
#' @param h,v Position of horizontal or vertical reference line
#' @param size Line size
#' @param colour Line colour
#' @export
geom_ref_line <- function(h, v, size = 2, colour = "white") {
  if (!missing(h)) {
    ggplot2::geom_hline(yintercept = h, size = size, colour = colour)
  } else if (!missing(v)) {
    ggplot2::geom_vline(xintercept = v, size = size, colour = colour)
  } else {
    stop("Must supply one of `h` and `v`.", call. = FALSE)
  }
}

#' Add residuals to a data frame
#'
#' @param data A data frame used to generate the residuals
#' @param model,var `add_residuals` takes a single `model`; the
#'   output column will be called `resid`
#' @param ... `gather_residuals` and `spread_residuals` take
#'   multiple models. The name will be taken from either the argument
#'   name of the name of the model.
#' @param .resid,.model The variable names used by `gather_residuals`.
#' @return A data frame. `add_residuals` adds a single new column,
#'   `.resid`, to the input `data`. `spread_residuals` adds
#'   one column for each model. `gather_predictions` adds two columns
#'   `.model` and `.resid`, and repeats the input rows for
#'   each model.
#' @export
#' @examples
#' df <- tibble::tibble(
#'   x = sort(runif(100)),
#'   y = 5 * x + 0.5 * x ^ 2 + 3 + rnorm(length(x))
#' )
#' plot(df)
#'
#' m1 <- lm(y ~ x, data = df)
#' df %>% add_residuals(m1)
#'
#' m2 <- lm(y ~ poly(x, 2), data = df)
#' df %>% spread_residuals(m1, m2)
#' df %>% gather_residuals(m1, m2)
add_residuals <- function(data, model, var = "resid") {
  data[[var]] <- residuals(model, data)
  data
}

#' @rdname add_residuals
#' @export
spread_residuals <- function(data, ...) {
  models <- tibble::lst(...)
  for (nm in names(models)) {
    data[[nm]] <- residuals(models[[nm]], data)
  }
  data
}

#' @rdname add_residuals
#' @export
gather_residuals <- function(data, ..., .resid = "resid", .model = "model") {
  models <- tibble::lst(...)

  df <- purrr::map2(models, .resid, add_residuals, data = data)
  names(df) <- names(models)

  vctrs::vec_rbind(!!!df, .names_to = .model)
}

residuals <- function(model, data) {
  response(model, data) - stats::predict(model, data)
}

test_that("*_residuals() return expected shapes", {
  df <- tibble::tibble(x = 1:5, y = c(1, 4, 3, 2, 5))
  m1 <- lm(y ~ x, data = df)
  m2 <- lm(y ~ poly(x, 2), data = df)

  out <- spread_residuals(df, m1, m2)
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("x", "y", "m1", "m2"))
  expect_equal(nrow(out), nrow(df))

  out <- gather_residuals(df, m1, m2)
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("model", "x", "y", "resid"))
  expect_equal(nrow(out), nrow(df) * 2)

  out <- add_residuals(df, m1)
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("x", "y", "resid"))
})

#' Generate an exclusive partitioning of a data frame
#'
#' @param data A data frame
#' @param p A named numeric vector giving where the value is the probability
#'   that an observation will be assigned to that group.
#' @family resampling techniques
#' @export
#' @examples
#' ex <- resample_partition(mtcars, c(test = 0.3, train = 0.7))
#' mod <- lm(mpg ~ wt, data = ex$train)
#' rmse(mod, ex$test)
#' rmse(mod, ex$train)
resample_partition <- function(data, p) {
  if (!is.numeric(p) || length(p) < 2 || !all(has_name(p))) {
    stop("`p` must be a named numeric vector with at least two values.")
  }

  if (abs(sum(p) - 1) > 1e-6) {
    message("Rescaling `p` to sum to 1.")
  }
  # Always rescale so sums exactly to 1
  p <- p / sum(p)

  n <- nrow(data)
  g <- findInterval(seq_len(n) / n, c(0, cumsum(p)), rightmost.closed = TRUE)

  idx <- split(seq_len(n), sample(g))
  names(idx) <- names(p)

  map(idx, resample, data = data)
}

test_that("resample_partition() generates error messages & warnings", {
  expect_error(
    resample_partition(mtcars, c(test = "0.8", train = 0.7)),
    "numeric vector"
  )
  expect_error(
    resample_partition(mtcars, c(train = 0.7)),
    "numeric vector"
  )
  expect_error(
    resample_partition(mtcars, c(0.3, 0.7)),
    "numeric vector"
  )
  expect_message(
    resample_partition(mtcars, c(test = 0.8, train = 0.7)),
    "sum to"
  )
})

test_that("resample_partition() returns a list", {
  out <- resample_partition(mtcars, c(test = 0.3, train = 0.7))
  expect_true(is.list(out))
})

find_symbols <- function(lang) {
  if (is.call(lang)) {
    find_symbols_call(lang)
  } else if (is.symbol(lang)) {
    as.character(lang)
  } else {
    character(0)
  }
}

find_symbols_call <- function(lang) {
  fun_name <- as.character(lang[[1]])

  if (fun_name %in% c("$", "@")) {
    as.character(lang[[2]])
  } else if (fun_name %in% c("::", ":::")) {
    character(0)
  } else {
    res <- map(as.list(lang[-1]), find_symbols)
    purrr::flatten_chr(res)
  }
}

f_zap_lhs <- function(f) {
  new_formula(NULL, f_rhs(f), env = f_env(f))
}

is_formula <- function(f) {
  inherits(f, "formula")
}

#' Fit a list of formulas
#'
#' `fit_with()` is a pipe-friendly tool that applies a list of
#' formulas to a fitting function such as [stats::lm()].
#' The list of formulas is typically created with [formulas()].
#'
#' Assumes that `.f` takes the formula either as first argument
#' or as second argument if the first argument is `data`.  Most
#' fitting functions should fit these requirements.
#' @param data A dataset used to fit the models.
#' @param .f A fitting function such as [stats::lm()],
#'   [lme4::lmer()] or [rstanarm::stan_glmer()].
#' @param .formulas A list of formulas specifying a model.
#' @param ... Additional arguments passed on to `.f`
#' @seealso [formulas()]
#' @export
#' @examples
#' # fit_with() is typically used with formulas().
#' disp_fits <- mtcars %>% fit_with(lm, formulas(~disp,
#'   additive = ~drat + cyl,
#'   interaction = ~drat * cyl,
#'   full = add_predictors(interaction, ~am, ~vs)
#' ))
#'
#' # The list of fitted models is named after the names of the list of
#' # formulas:
#' disp_fits$full
#'
#' # Additional arguments are passed on to .f
#' mtcars %>% fit_with(glm, list(am ~ disp), family = binomial)
fit_with <- function(data, .f, .formulas, ...) {
  args <- list(...)

  # Supply .f quoted, otherwise the whole fitting function is inlined
  # in the call recorded in the fit objects
  map(.formulas, function(formula) {
    inject(.f(!!formula, data = data, !!!args))
  })
}

#' Create a list of formulas
#'
#' `formulas()` creates a list of two-sided formulas by merging a
#' unique left-hand side to a list of right-hand sides.
#' @param .response A one-sided formula used as the left-hand side of
#'   all resulting formulas.
#' @param ... List of formulas whose right-hand sides will be merged
#'   to `.response`.
#' @export
#' @examples
#' # Provide named arguments to create a named list of formulas:
#' models <- formulas(~lhs,
#'   additive = ~var1 + var2,
#'   interaction = ~var1 * var2
#' )
#' models$additive
#'
#' # The formulas are created sequentially, so that you can refer to
#' # previously created formulas:
#' formulas(~lhs,
#'   linear = ~var1 + var2,
#'   hierarchical = add_predictors(linear, ~(1 | group))
#' )
formulas <- function(.response, ...) {
  formulas <- tibble::lst(...)
  validate_formulas(.response, formulas)
  map(formulas, set_lhs, .response)
}

#' @rdname formulas
#' @export
formulae <- formulas

validate_formulas <- function(response, formulas) {
  if (!is_formula(response) || length(response) != 2) {
    stop(".response must be a one-sided formula", call. = FALSE)
  }

  if (!length(formulas)) {
    stop("No formula provided", call. = FALSE)
  }
  purrr::walk(formulas, function(f) {
    if (!is_formula(f)) {
      stop("'...' must contain only formulas", call. = FALSE)
    }
  })
}

set_lhs <- function(f, lhs) {
  env <- merge_envs(lhs, f)
  new_formula(f_rhs(lhs), f_rhs(f), env)
}

#' Add predictors to a formula
#'
#' This merges a one- or two-sided formula `f` with the
#' right-hand sides of all formulas supplied in `...`.
#' @param f A formula.
#' @param ... Formulas whose right-hand sides will be merged to
#'   `f`.
#' @param fun A function name indicating how to merge the right-hand
#'   sides.
#' @export
#' @examples
#' f <- lhs ~ rhs
#' add_predictors(f, ~var1, ~var2)
#'
#' # Left-hand sides are ignored:
#' add_predictors(f, lhs1 ~ var1, lhs2 ~ var2)
#'
#' # fun can also be set to a function like "*":
#' add_predictors(f, ~var1, ~var2, fun = "*")
add_predictors <- function(f, ..., fun = "+") {
  rhss <- map(list(f, ...), f_zap_lhs)
  rhs <- reduce(rhss, merge_formulas, fun = fun)
  env <- merge_envs(f, rhs)

  new_formula(f_lhs(f), f_rhs(rhs), env)
}

merge_formulas <- function(f1, f2, fun = "+") {
  rhs <- call(fun, f_rhs(f1), f_rhs(f2))

  lhss <- compact(map(list(f1, f2), f_lhs))
  if (length(lhss) == 0) {
    lhs <- NULL
  } else {
    lhs <- reduce_common(lhss, "Left-hand sides must be identical")
  }

  env <- merge_envs(f1, f2)
  new_formula(lhs, rhs, env)
}

merge_envs <- function(f1, f2) {
  symbols_f1 <- find_symbols(f1)
  symbols_f2 <- find_symbols(f2)

  conflicts <- intersect(symbols_f1, symbols_f2)
  conflicts_envs <- compact(map(conflicts, find_env_conflicts, f1, f2))

  all_symbols <- union(symbols_f1, symbols_f2)
  nonconflicts <- setdiff(all_symbols, conflicts)
  nonconflicts_envs <- compact(map(nonconflicts, find_env_nonconflicts, f1, f2))

  all_envs <- c(conflicts_envs, nonconflicts_envs)

  if (length(all_envs) == 0) {
    environment(f1)
  } else {
    reduce_common(
      all_envs,
      "Cannot merge formulas as their scopes conflict across symbols"
    )
  }
}

find_env_conflicts <- function(symbol, f1, f2) {
  env1 <- find_binding_env(symbol, environment(f1))
  env2 <- find_binding_env(symbol, environment(f2))

  if (is.null(env1) || is.null(env2)) {
    return(env1 %||% env2)
  }

  if (!identical(env1, env2)) {
    stop("Cannot merge formulas as their scopes conflict for the symbol '",
      symbol, "'",
      call. = FALSE
    )
  }

  env1
}

find_env_nonconflicts <- function(symbol, f1, f2) {
  env1 <- find_binding_env(symbol, environment(f1))
  env2 <- find_binding_env(symbol, environment(f2))
  env1 %||% env2
}

find_binding_env <- function(symbol, env) {
  if (exists(symbol, envir = env)) {
    env
  } else if (!identical(env, emptyenv())) {
    find_binding_env(symbol, parent.env(env))
  } else {
    NULL
  }
}

test_that("add_predictors() combines predictors", {
  expect_identical(add_predictors(~1, ~2, ~3), ~1 + 2 + 3)
})

test_that("add_predictors() combines with fun", {
  expect_identical(add_predictors(~1, ~2, ~3, fun = "*"), ~1 * 2 * 3)
})

test_that("add_predictors() handles lhss", {
  expect_identical(add_predictors(lhs ~ 1, ~2), lhs ~ 1 + 2)
  expect_identical(add_predictors(lhs1 ~ 1, lhs2 ~ 2), lhs1 ~ 1 + 2)
})

test_that("merge_formula() handles lhss", {
  expect_identical(merge_formulas(lhs ~ rhs, lhs ~ rhs), lhs ~ rhs + rhs)
  expect_error(merge_formulas(lhs ~ rhs, other_lhs ~ rhs), "must be identical")
})

test_that("merging formulas fail when scope conflicts within symbols", {
  env <- new.env(parent = emptyenv())
  env$object <- list()
  object <- list()

  f_conflict <- new_formula(NULL, quote(object), env = env)
  expect_error(merge_envs(~object, f_conflict), "conflict for the symbol 'object'")
})

test_that("merging formulas fail when scope conflicts between symbols", {
  env1 <- new.env(parent = emptyenv())
  env1$object <- list()
  env2 <- new.env(parent = emptyenv())
  env2$other_object <- list()
  f1 <- new_formula(NULL, quote(list(object)), env = env1)
  f2 <- new_formula(NULL, quote(list(other_object)), env = env2)
  expect_error(merge_envs(f1, f2), "conflict across symbols")
})


test_that("formulas() fails when supplied non-formula objects", {
  expect_error(formulas(~lhs, NULL), "must contain only formulas")
})

test_that("formulas() combines the lhs", {
  expect_equal(formulas(~lhs, a = ~1, b = other ~ 2), list(a = lhs ~ 1, b = lhs ~ 2))
})

test_that("bytecoded fit_with() works", {
  bc_fit_with <- compiler::cmpfun(fit_with)
  fit <- bc_fit_with(mtcars, lm, list(disp ~ drat))
  expect_s3_class(fit[[1]], "lm")
})

response_var <- function(model) {
  UseMethod("response_var")
}

#' @export
response_var.default <- function(model, data) {
  stats::formula(model)[[2L]]
}

predictor_vars <- function(model) {
  UseMethod("predictor_vars")
}

#' @export
predictor_vars.default <- function(model, data) {
  all.vars(stats::formula(model)[[3L]])
}

response <- function(model, data) {
  eval(response_var(model), as.data.frame(data))
}

test_that("can extract response from linear model", {
  mod <- lm(mpg ~ wt, data = mtcars)
  expect_identical(response_var(mod), quote(mpg))
})

test_that("can extract response from loess model", {
  mod <- loess(mpg ~ wt, data = mtcars)
  expect_identical(response_var(mod), quote(mpg))
})

#' Generate a data grid.
#'
#' To visualise a model, it is very useful to be able to generate an
#' evenly spaced grid of points from the data. `data_grid` helps you
#' do this by wrapping around [tidyr::expand()].
#'
#' @param data A data frame
#' @param ... Variables passed on to [tidyr::expand()]
#' @param .model A model.  If supplied, any predictors needed for the model
#'   not present in `...` will be filled in with "\link{typical}" values.
#' @export
#' @seealso [seq_range()] for generating ranges from continuous
#'   variables.
#' @examples
#' data_grid(mtcars, vs, am)
#'
#' # For continuous variables, seq_range is useful
#' data_grid(mtcars, mpg = mpg)
#' data_grid(mtcars, mpg = seq_range(mpg, 10))
#'
#' # If you supply a model, missing predictors will be filled in with
#' # typical values
#' mod <- lm(mpg ~ wt + cyl + vs, data = mtcars)
#' data_grid(mtcars, .model = mod)
#' data_grid(mtcars, cyl = seq_range(cyl, 9), .model = mod)
data_grid <- function(data, ..., .model = NULL) {
  if (missing(...) && missing(.model)) {
    abort("Must supply at least one of `...` and `.model`")
  }

  if (missing(...)) {
    expanded <- NULL
  } else {
    expanded <- tidyr::expand(data, ...)
  }

  if (is.null(.model)) {
    return(expanded)
  }

  # Generate grid of typical values
  needed <- setdiff(predictor_vars(.model), names(expanded))
  if (length(needed) > 0) {
    typical_vals <- lapply(data[needed], typical)
    typical_df <- tidyr::crossing(!!!typical_vals)
  } else {
    typical_df <- NULL
  }

  tidyr::crossing(expanded, typical_df)
}

test_that("can generate typical values", {
  df <- tibble::tibble(
    x = rep(c("a", "b"), each = 5),
    y = rep(c("a", "b"), 5),
    z = rnorm(10)
  )
  mod <- lm(z ~ x + y, data = df)
  out <- data_grid(df, .model = mod)

  expect_equal(out$x, c("a", "a", "b", "b"))
  expect_equal(out$y, c("a", "b", "a", "b"))
})

test_that("data_grid() returns a tibble", {
  mod <- lm(mpg ~ wt, data = mtcars)
  out <- data_grid(mtcars, .model = mod)
  expect_s3_class(out, "tbl_df")
})

#' Handle missing values with a warning
#'
#' This NA handler ensures that those models that support the
#' `na.action` parameter do not silently drop missing values. It
#' wraps around [stats::na.exclude()] so that there is one
#' prediction/residual for input row. To apply it globally, run
#' `options(na.action = na.warn)`.
#'
#' @param object A data frame
#' @export
#' @examples
#' df <- tibble::tibble(
#'   x = 1:10,
#'   y = c(5.1, 9.7, NA, 17.4, 21.2, 26.6, 27.9, NA, 36.3, 40.4)
#' )
#' # Default behaviour is to silently drop
#' m1 <- lm(y ~ x, data = df)
#' resid(m1)
#'
#' # Use na.action = na.warn to warn
#' m2 <- lm(y ~ x, data = df, na.action = na.warn)
#' resid(m2)
na.warn <- function(object) {
  missing <- sum(!stats::complete.cases(object))
  if (missing > 0) {
    warning("Dropping ", missing, " rows with missing values", call. = FALSE)
  }

  stats::na.exclude(object)
}

#' Generate `n` permutation replicates.
#'
#' A permutation test involves permuting one or more variables in a data set
#' before performing the test, in order to break any existing relationships
#' and simulate the null hypothesis. One can then compare the true statistic
#' to the generated distribution of null statistics.
#'
#' @inheritParams resample_partition
#' @param n Number of permutations to generate.
#' @param ... Columns to permute. This supports bare column names or dplyr
#' [dplyr::select_helpers]
#' @param columns In `permute_`, vector of column names to permute.
#' @param .id Name of variable that gives each model a unique integer id.
#'
#' @return A data frame with `n` rows and one column: `perm`
#' @export
#' @examples
#'
#' library(purrr)
#' perms <- permute(mtcars, 100, mpg)
#'
#' models <- map(perms$perm, ~ lm(mpg ~ wt, data = .))
#' glanced <- map_df(models, broom::glance, .id = "id")
#'
#' # distribution of null permutation statistics
#' hist(glanced$statistic)
#' # confirm these are roughly uniform p-values
#' hist(glanced$p.value)
#'
#' # test against the unpermuted model to get a permutation p-value
#' mod <- lm(mpg ~ wt, mtcars)
#' mean(glanced$statistic > broom::glance(mod)$statistic)
#'
#' @export
permute <- function(data, n, ..., .id = ".id") {
  columns <- tidyselect::vars_select(colnames(data), ...)
  permute_(data, n, columns, .id = .id)
}

#' @rdname permute
#' @export
permute_ <- function(data, n, columns, .id = ".id") {
  perm <- purrr::rerun(n, resample_permutation(data, columns))

  df <- tibble::tibble(perm = perm)
  df[[.id]] <- id(n)
  df
}

#' Create a resampled permutation of a data frame
#'
#' @param data A data frame
#' @param columns Columns to be permuted
#' @param idx Indices to permute by. If not given, generates them randomly
#'
#' @return A permutation object; use as.data.frame to convert to a permuted
#' data frame
#'
#' @export
resample_permutation <- function(data, columns, idx = NULL) {
  if (is.null(idx)) {
    idx <- sample.int(nrow(data))
  }

  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }
  if (!is.character(columns) ||
    !(all(columns %in% colnames(data)))) {
    stop("`columns` must be a vector of column names in `data`", call. = FALSE)
  }
  if (!is.integer(idx) || length(idx) != nrow(data)) {
    stop("`idx` must be an integer vector with the same length as there are ",
      "rows in `data`",
      call. = FALSE
    )
  }

  structure(
    list(
      data = data,
      columns = columns,
      idx = idx
    ),
    class = "permutation"
  )
}

#' @export
print.permutation <- function(x, ...) {
  n <- length(x$idx)
  if (n > 10) {
    id10 <- c(x$idx[1:10], "...")
  } else {
    id10 <- x$idx
  }

  cat("<", obj_sum.permutation(x), "> ", paste(id10, collapse = ", "), "\n",
    sep = ""
  )
}

#' @export
as.integer.permutation <- function(x, ...) {
  x$idx
}

#' @export
as.data.frame.permutation <- function(x, ...) {
  ret <- x$data
  indices <- x$idx
  for (col in x$columns) {
    ret[[col]] <- ret[[col]][indices]
  }
  ret
}

#' @export
dim.permutation <- function(x, ...) {
  c(length(x$idx), ncol(x$data))
}

#' @importFrom tibble obj_sum
#' @method obj_sum resample
#' @export
obj_sum.permutation <- function(x, ...) {
  paste0(
    "permutation (", paste0(x$columns, collapse = ", "), ") [",
    big_mark(nrow(x)), " x ", big_mark(ncol(x)), "]"
  )
}

#' Construct a design matrix
#'
#' This is a thin wrapper around [stats::model.matrix()] which
#' returns a tibble. Use it to determine how your modelling formula is
#' translated into a matrix, an thence into an equation.
#'
#' @param data A data frame
#' @param formula A modelling formula
#' @param ... Other arguments passed onto [stats::model.matrix()]
#' @return A tibble.
#' @export
#' @examples
#' model_matrix(mtcars, mpg ~ cyl)
#' model_matrix(iris, Sepal.Length ~ Species)
#' model_matrix(iris, Sepal.Length ~ Species - 1)
model_matrix <- function(data, formula, ...) {
  out <- stats::model.matrix(formula, data = data, ...)
  mostattributes(out) <- NULL

  tibble::as_tibble(out)
}

#' Pipe operator
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
NULL

reduce_common <- function(x, msg = "Objects must be identical",
                          operator = identical) {
  reduce(x, function(.x, .y) {
    if (!operator(.x, .y)) {
      stop(msg, call. = FALSE)
    }
    .y
  })
}


has_name <- function(x) {
  nms <- names(x)
  if (is.null(nms)) {
    rep(FALSE, length(x))
  } else {
    !(is.na(nms) | nms == "")
  }
}

big_mark <- function(x, ...) {
  mark <- if (identical(getOption("OutDec"), ",")) "." else ","
  formatC(x, big.mark = mark, ...)
}

id <- function(n) {
  width <- nchar(n)
  sprintf(paste0("%0", width, "d"), seq_len(n))
}

#' @importFrom broom tidy
NULL

#' Generate `n` bootstrap replicates.
#'
#' @inheritParams resample_partition
#' @family resampling techniques
#' @param n Number of bootstrap replicates to generate
#' @param id Name of variable that gives each model a unique integer id.
#' @return A data frame with `n` rows and one column: `strap`
#' @export
#' @examples
#' library(purrr)
#' boot <- bootstrap(mtcars, 100)
#'
#' models <- map(boot$strap, ~ lm(mpg ~ wt, data = .))
#' tidied <- map_df(models, broom::tidy, .id = "id")
#'
#' hist(subset(tidied, term == "wt")$estimate)
#' hist(subset(tidied, term == "(Intercept)")$estimate)
bootstrap <- function(data, n, id = ".id") {
  bootstrap <- purrr::rerun(n, resample_bootstrap(data))

  df <- tibble::tibble(strap = bootstrap)
  df[[id]] <- id(n)
  df
}

#' Generate a boostrap replicate
#'
#' @param data A data frame
#' @family resampling techniques
#' @export
#' @examples
#' coef(lm(mpg ~ wt, data = resample_bootstrap(mtcars)))
#' coef(lm(mpg ~ wt, data = resample_bootstrap(mtcars)))
#' coef(lm(mpg ~ wt, data = resample_bootstrap(mtcars)))
resample_bootstrap <- function(data) {
  resample(data, sample(nrow(data), replace = TRUE))
}

#' Height and income data.
#'
#' You might have heard that taller people earn more. Is it true? You can
#' try and answer the question by exploring this dataset extracted from the
#' \href{https://www.nlsinfo.org}{National Longitudinal Study}, which is
#' sponsored by the U.S. Bureau of Labor Statistics.
#'
#' This contains data as at 2012.
#' @format
#' \describe{
#'  \item{income}{Yearly income. The top two percent of values were averaged
#'    and that average was used to replace all values in the top range. }
#'  \item{height}{Height, in inches}
#'  \item{weight}{Weight, in pounds}
#'  \item{age}{Age, in years, between 47 and 56.}
#'  \item{marital}{Marital status}
#'  \item{sex}{Sex}
#'  \item{education}{Years of education}
#'  \item{afqt}{Percentile score on Armed Forces Qualification Test.}
#' }
"heights"


#' Simple simulated datasets
#'
#' These simple simulated datasets are useful for teaching modelling
#' basics.
#'
#' @name sim
#' @format NULL
NULL

#' @rdname sim
#' @format NULL
"sim1"

#' @rdname sim
#' @format NULL
"sim2"

#' @rdname sim
#' @format NULL
"sim3"

#' @rdname sim
#' @format NULL
"sim4"

#' Generate a sequence over the range of a vector
#'
#' @param x A numeric vector
#' @param n,by Specify the output sequence either by supplying the
#'   length of the sequence with `n`, or the spacing between value
#'   with `by`. Specifying both is an error.
#'
#'   I recommend that you name these arguments in order to make it clear to
#'   the reader.
#' @param pretty If `TRUE`, will generate a pretty sequence. If `n`
#'   is supplied, this will use [pretty()] instead of
#'   [seq()]. If `by` is supplied, it will round the first
#'   value to a multiple of `by`.
#' @param trim Optionally, trim values off the tails.
#'   `trim / 2 * length(x)` values are removed from each tail.
#' @param expand Optionally, expand the range by `expand * (1 + range(x)`
#'   (computed after trimming).
#' @export
#' @examples
#' x <- rcauchy(100)
#' seq_range(x, n = 10)
#' seq_range(x, n = 10, trim = 0.1)
#' seq_range(x, by = 1, trim = 0.1)
#'
#' # Make pretty sequences
#' y <- runif(100)
#' seq_range(y, n = 10)
#' seq_range(y, n = 10, pretty = TRUE)
#' seq_range(y, n = 10, expand = 0.5, pretty = TRUE)
#'
#' seq_range(y, by = 0.1)
#' seq_range(y, by = 0.1, pretty = TRUE)
seq_range <- function(x, n, by, trim = NULL, expand = NULL, pretty = FALSE) {
  if (!missing(n) && !missing(by)) {
    stop("May only specify one of `n` and `by`", call. = FALSE)
  }

  if (!is.null(trim)) {
    rng <- stats::quantile(x, c(trim / 2, 1 - trim / 2), na.rm = TRUE)
  } else {
    rng <- range(x, na.rm = TRUE)
  }

  if (!is.null(expand)) {
    rng <- rng + c(-expand / 2, expand / 2) * (rng[2] - rng[1])
  }

  if (missing(by)) {
    if (pretty) {
      pretty(rng, n)
    } else {
      seq(rng[1], rng[2], length.out = n)
    }
  } else {
    if (pretty) {
      rng[1] <- floor(rng[1] / by) * by
      rng[2] <- ceiling(rng[2] / by) * by
    }
    seq(rng[1], rng[2], by = by)
  }
}

test_that("seq_range() can generate error messages and typical values", {
  x <- 1:4
  expect_error(
    seq_range(x, by = 0.1, n = 10, expand = 0.5, pretty = TRUE),
    "one of"
  )
  expect_equal(seq_range(x, n = 4), c(1, 2, 3, 4))

  out <- seq_range(x, n = 3, expand = 0.5, pretty = TRUE)
  expect_equal(out, c(0, 2, 4, 6))

  out <- seq_range(x, by = 150, expand = 0.5, pretty = TRUE)
  expect_equal(out, c(0, 150))
})

#' Find the typical value
#'
#' For numeric, integer, and ordered factor vectors, it returns the median.
#' For factors, characters, and logical vectors, it returns the most
#' frequent value. If multiple values are tied for most frequent, it returns
#' them all. `NA` missing values are always silently dropped.
#'
#' @param x A vector
#' @param ... Arguments used by methods
#' @export
#' @importFrom stats quantile
#' @examples
#' # median of numeric vector
#' typical(rpois(100, lambda = 10))
#'
#' # most frequent value of character or factor
#' x <- sample(c("a", "b", "c"), 100, prob = c(0.6, 0.2, 0.2), replace = TRUE)
#' typical(x)
#' typical(factor(x))
#'
#' # if tied, returns them all
#' x <- c("a", "a", "b", "b", "c")
#' typical(x)
#'
#' # median of an ordered factor
#' typical(ordered(c("a", "a", "b", "c", "d")))
#'
typical <- function(x, ...) {
  UseMethod("typical")
}

#' @export
typical.numeric <- function(x, ...) {
  stats::median(x, na.rm = TRUE)
}

#' @export
typical.factor <- function(x, ...) {
  counts <- table(x, exclude = NULL)
  levels(x)[max(counts) == counts]
}

#' @export
typical.character <- function(x, ...) {
  counts <- table(x, exclude = NULL)
  names(counts)[max(counts) == counts]
}

#' @export
typical.logical <- function(x, ...) {
  mean(x, na.rm = TRUE) >= 0.5
}

#' @export
typical.integer <- function(x, ...) {
  unname(stats::quantile(x, 0.5, type = 1, na.rm = TRUE))
}

#' @export
typical.ordered <- function(x, ...) {
  levels(x)[stats::quantile(as.integer(x), 0.5, type = 1, na.rm = TRUE)]
}

test_that("typical.numeric works as expected", {
  x <- c(1, 2, 5)
  expect_equal(typical(x), median(x))
  expect_equal(typical(c(x, NA)), median(x))
})

test_that("typical.character works as exepected", {
  expect_equal(typical(c("a", "a", "b")), "a")
  expect_equal(typical(c("a", "a", "b", "b", NA_character_)), c("a", "b"))
  expect_equal(typical(c("a", "a", "b", "b")), c("a", "b"))
})

test_that("NA values are considered typical", {
  x <- c(NA, NA, NA, "x")
  expect_equal(typical(x), NA_character_)

  f <- factor(x, exclude = NULL)
  expect_equal(typical(f), NA_character_)
})

test_that("typical.factor works as exepected", {
  expect_equal(typical(factor(c("a", "a", "b"))), "a")
  expect_equal(typical(factor(c("a", "a", "b", NA))), "a")
  expect_equal(typical(factor(c("a", "a", "b", "b"))), c("a", "b"))
})

test_that("typical.logical works as exepected", {
  expect_equal(typical(c(TRUE, FALSE, TRUE)), TRUE)
  expect_equal(typical(c(TRUE, TRUE, FALSE, FALSE)), TRUE)
  expect_equal(typical(c(TRUE, FALSE, TRUE, NA)), TRUE)
})

test_that("typical.ordered works as expected", {
  expect_identical(typical(ordered(c("d", "d", "a", "b"))), "b")
  expect_identical(typical(ordered(c("d", "d", "a", "b", NA))), "b")
})

test_that("typical.integer works as expected", {
  expect_equal(typical(c(1L, 2L, 3L, 8L)), 2)
  expect_equal(typical(c(1L, 2L, 3L, 8L, NA_integer_)), 2)
})

#' Add predictions to a data frame
#'
#' @param data A data frame used to generate the predictions.
#' @param model `add_predictions` takes a single `model`;
#' @param var The name of the output column, default value is `pred`
#' @param type Prediction type, passed on to `stats::predict()`. Consult
#'   `predict()` documentation for given `model` to determine valid values.
#' @param ... `gather_predictions` and `spread_predictions` take
#'   multiple models. The name will be taken from either the argument
#'   name of the name of the model.
#' @param .pred,.model The variable names used by `gather_predictions`.
#' @return A data frame. `add_prediction` adds a single new column,
#'   with default name `pred`, to the input `data`.
#'   `spread_predictions` adds one column for each model. `gather_predictions`
#'   adds two columns `.model` and `.pred`, and repeats the input rows for each
#'   model.
#' @export
#' @examples
#' df <- tibble::tibble(
#'   x = sort(runif(100)),
#'   y = 5 * x + 0.5 * x ^ 2 + 3 + rnorm(length(x))
#' )
#' plot(df)
#'
#' m1 <- lm(y ~ x, data = df)
#' grid <- data.frame(x = seq(0, 1, length = 10))
#' grid %>% add_predictions(m1)
#'
#' m2 <- lm(y ~ poly(x, 2), data = df)
#' grid %>% spread_predictions(m1, m2)
#' grid %>% gather_predictions(m1, m2)
add_predictions <- function(data, model, var = "pred", type = NULL) {
  data[[var]] <- predict2(model, data, type = type)
  data
}

#' @rdname add_predictions
#' @export
spread_predictions <- function(data, ..., type = NULL) {
  models <- tibble::lst(...)
  for (nm in names(models)) {
    data[[nm]] <- predict2(models[[nm]], data, type = type)
  }
  data
}

#' @rdname add_predictions
#' @export
gather_predictions <- function(data, ..., .pred = "pred", .model = "model", type = NULL) {

  models <- tibble::lst(...)

  df <- purrr::map2(models, .pred, add_predictions, data = data, type = type)
  names(df) <- names(models)

  vctrs::vec_rbind(!!!df, .names_to = .model)
}

predict2 <- function(model, data, type = NULL) {
  if (is.null(type)) {
    stats::predict(model, data)
  } else {
    stats::predict(model, data, type = type)
  }
}

test_that("always uses na.action = na.exclude", {
  df <- tibble::tibble(x = c(1, 2, NA), y = c(1.5, 2, 3.5))

  mod <- lm(y ~ x, data = df)

  out <- add_predictions(df, mod)

  expect_equal(2 * 2, 4)
})

test_that("*_predictions() return expected shapes", {
  df <- tibble::tibble(x = 1:5, y = c(1, 4, 3, 2, 5))
  mod <- lm(y ~ x, data = df)

  out <- add_predictions(df, mod)
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("x", "y", "pred"))

  out <- spread_predictions(df, m1 = mod, m2 = mod)
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("x", "y", "m1", "m2"))
  expect_equal(nrow(out), nrow(df))

  out <- gather_predictions(df, m1 = mod, m2 = mod)
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("model", "x", "y", "pred"))
  expect_equal(nrow(out), nrow(df) * 2)
})

#' A "lazy" resample.
#'
#' Often you will resample a dataset hundreds or thousands of times. Storing
#' the complete resample each time would be very inefficient so this class
#' instead stores a "pointer" to the original dataset, and a vector of row
#' indexes. To turn this into a regular data frame, call `as.data.frame`,
#' to extract the indices, use `as.integer`.
#'
#' @param data The data frame
#' @param idx A vector of integer indexes indicating which rows have
#'   been selected. These values should lie between 1 and `nrow(data)`
#'   but they are not checked by this function in the interests of performance.
#' @family resampling techniques
#' @export
#' @examples
#' resample(mtcars, 1:10)
#'
#' b <- resample_bootstrap(mtcars)
#' b
#' as.integer(b)
#' as.data.frame(b)
#'
#' # Many modelling functions will do the coercion for you, so you can
#' # use a resample object directly in the data argument
#' lm(mpg ~ wt, data = b)
resample <- function(data, idx) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }
  if (!is.integer(idx)) {
    stop("`idx` must be an integer vector.", call. = FALSE)
  }

  structure(
    list(
      data = data,
      idx = idx
    ),
    class = "resample"
  )
}

#' @export
print.resample <- function(x, ...) {
  n <- length(x$idx)
  if (n > 10) {
    id10 <- c(x$idx[1:10], "...")
  } else {
    id10 <- x$idx
  }

  cat("<", obj_sum.resample(x), "> ", paste(id10, collapse = ", "), "\n",
    sep = ""
  )
}

#' @export
as.integer.resample <- function(x, ...) {
  x$idx
}

#' @export
as.data.frame.resample <- function(x, ...) {
  x$data[x$idx, , drop = FALSE]
}

#' @export
dim.resample <- function(x, ...) {
  c(length(x$idx), ncol(x$data))
}

#' @importFrom tibble obj_sum
#' @method obj_sum resample
#' @export
obj_sum.resample <- function(x, ...) {
  paste0("resample [", big_mark(nrow(x)), " x ", big_mark(ncol(x)), "]")
}

test_that("na.warn warns when dropping", {
  df <- data.frame(x = 1:5, y = c(1, NA, 3, NA, 5))
  expect_warning(mod <- lm(y ~ x, data = df, na.action = na.warn), "Dropping 2")
  pred <- unname(predict(mod))
  expect_equal(is.na(pred), is.na(df$y))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formulas.R
\name{fit_with}
\alias{fit_with}
\title{Fit a list of formulas}
\usage{
fit_with(data, .f, .formulas, ...)
}
\arguments{
\item{data}{A dataset used to fit the models.}

\item{.f}{A fitting function such as \code{\link[stats:lm]{stats::lm()}},
\code{\link[lme4:lmer]{lme4::lmer()}} or \code{\link[rstanarm:stan_glmer]{rstanarm::stan_glmer()}}.}

\item{.formulas}{A list of formulas specifying a model.}

\item{...}{Additional arguments passed on to \code{.f}}
}
\description{
\code{fit_with()} is a pipe-friendly tool that applies a list of
formulas to a fitting function such as \code{\link[stats:lm]{stats::lm()}}.
The list of formulas is typically created with \code{\link[=formulas]{formulas()}}.
}
\details{
Assumes that \code{.f} takes the formula either as first argument
or as second argument if the first argument is \code{data}.  Most
fitting functions should fit these requirements.
}
\examples{
# fit_with() is typically used with formulas().
disp_fits <- mtcars \%>\% fit_with(lm, formulas(~disp,
  additive = ~drat + cyl,
  interaction = ~drat * cyl,
  full = add_predictors(interaction, ~am, ~vs)
))

# The list of fitted models is named after the names of the list of
# formulas:
disp_fits$full

# Additional arguments are passed on to .f
mtcars \%>\% fit_with(glm, list(am ~ disp), family = binomial)
}
\seealso{
\code{\link[=formulas]{formulas()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_ref_line.R
\name{geom_ref_line}
\alias{geom_ref_line}
\title{Add a reference line (ggplot2).}
\usage{
geom_ref_line(h, v, size = 2, colour = "white")
}
\arguments{
\item{h, v}{Position of horizontal or vertical reference line}

\item{size}{Line size}

\item{colour}{Line colour}
}
\description{
Add a reference line (ggplot2).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/typical.R
\name{typical}
\alias{typical}
\title{Find the typical value}
\usage{
typical(x, ...)
}
\arguments{
\item{x}{A vector}

\item{...}{Arguments used by methods}
}
\description{
For numeric, integer, and ordered factor vectors, it returns the median.
For factors, characters, and logical vectors, it returns the most
frequent value. If multiple values are tied for most frequent, it returns
them all. \code{NA} missing values are always silently dropped.
}
\examples{
# median of numeric vector
typical(rpois(100, lambda = 10))

# most frequent value of character or factor
x <- sample(c("a", "b", "c"), 100, prob = c(0.6, 0.2, 0.2), replace = TRUE)
typical(x)
typical(factor(x))

# if tied, returns them all
x <- c("a", "a", "b", "b", "c")
typical(x)

# median of an ordered factor
typical(ordered(c("a", "a", "b", "c", "d")))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seq_range.R
\name{seq_range}
\alias{seq_range}
\title{Generate a sequence over the range of a vector}
\usage{
seq_range(x, n, by, trim = NULL, expand = NULL, pretty = FALSE)
}
\arguments{
\item{x}{A numeric vector}

\item{n, by}{Specify the output sequence either by supplying the
length of the sequence with \code{n}, or the spacing between value
with \code{by}. Specifying both is an error.

I recommend that you name these arguments in order to make it clear to
the reader.}

\item{trim}{Optionally, trim values off the tails.
\code{trim / 2 * length(x)} values are removed from each tail.}

\item{expand}{Optionally, expand the range by \verb{expand * (1 + range(x)}
(computed after trimming).}

\item{pretty}{If \code{TRUE}, will generate a pretty sequence. If \code{n}
is supplied, this will use \code{\link[=pretty]{pretty()}} instead of
\code{\link[=seq]{seq()}}. If \code{by} is supplied, it will round the first
value to a multiple of \code{by}.}
}
\description{
Generate a sequence over the range of a vector
}
\examples{
x <- rcauchy(100)
seq_range(x, n = 10)
seq_range(x, n = 10, trim = 0.1)
seq_range(x, by = 1, trim = 0.1)

# Make pretty sequences
y <- runif(100)
seq_range(y, n = 10)
seq_range(y, n = 10, pretty = TRUE)
seq_range(y, n = 10, expand = 0.5, pretty = TRUE)

seq_range(y, by = 0.1)
seq_range(y, by = 0.1, pretty = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resample.R
\name{resample}
\alias{resample}
\title{A "lazy" resample.}
\usage{
resample(data, idx)
}
\arguments{
\item{data}{The data frame}

\item{idx}{A vector of integer indexes indicating which rows have
been selected. These values should lie between 1 and \code{nrow(data)}
but they are not checked by this function in the interests of performance.}
}
\description{
Often you will resample a dataset hundreds or thousands of times. Storing
the complete resample each time would be very inefficient so this class
instead stores a "pointer" to the original dataset, and a vector of row
indexes. To turn this into a regular data frame, call \code{as.data.frame},
to extract the indices, use \code{as.integer}.
}
\examples{
resample(mtcars, 1:10)

b <- resample_bootstrap(mtcars)
b
as.integer(b)
as.data.frame(b)

# Many modelling functions will do the coercion for you, so you can
# use a resample object directly in the data argument
lm(mpg ~ wt, data = b)
}
\seealso{
Other resampling techniques: 
\code{\link{bootstrap}()},
\code{\link{resample_bootstrap}()},
\code{\link{resample_partition}()}
}
\concept{resampling techniques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap}
\alias{bootstrap}
\title{Generate \code{n} bootstrap replicates.}
\usage{
bootstrap(data, n, id = ".id")
}
\arguments{
\item{data}{A data frame}

\item{n}{Number of bootstrap replicates to generate}

\item{id}{Name of variable that gives each model a unique integer id.}
}
\value{
A data frame with \code{n} rows and one column: \code{strap}
}
\description{
Generate \code{n} bootstrap replicates.
}
\examples{
library(purrr)
boot <- bootstrap(mtcars, 100)

models <- map(boot$strap, ~ lm(mpg ~ wt, data = .))
tidied <- map_df(models, broom::tidy, .id = "id")

hist(subset(tidied, term == "wt")$estimate)
hist(subset(tidied, term == "(Intercept)")$estimate)
}
\seealso{
Other resampling techniques: 
\code{\link{resample_bootstrap}()},
\code{\link{resample_partition}()},
\code{\link{resample}()}
}
\concept{resampling techniques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross-validation.R
\name{crossv_mc}
\alias{crossv_mc}
\alias{crossv_kfold}
\alias{crossv_loo}
\title{Generate test-training pairs for cross-validation}
\usage{
crossv_mc(data, n, test = 0.2, id = ".id")

crossv_kfold(data, k = 5, id = ".id")

crossv_loo(data, id = ".id")
}
\arguments{
\item{data}{A data frame}

\item{n}{Number of test-training pairs to generate (an integer).}

\item{test}{Proportion of observations that should be held out for testing
(a double).}

\item{id}{Name of variable that gives each model a unique integer id.}

\item{k}{Number of folds (an integer).}
}
\value{
A data frame with columns \code{test}, \code{train}, and \code{.id}.
\code{test} and \code{train} are list-columns containing \code{\link[=resample]{resample()}} objects.
The number of rows is \code{n} for \code{crossv_mc()}, \code{k} for \code{crossv_kfold()}
and \code{nrow(data)} for \code{crossv_loo()}.
}
\description{
\code{crossv_kfold} splits the data into \code{k} exclusive partitions,
and uses each partition for a test-training split. \code{crossv_mc}
generates \code{n} random partitions, holding out \code{test} of the
data for training. \code{crossv_loo} performs leave-one-out
cross-validation, i.e., \code{n = nrow(data)} training partitions containing
\code{n - 1} rows each.
}
\examples{
cv1 <- crossv_kfold(mtcars, 5)
cv1

library(purrr)
cv2 <- crossv_mc(mtcars, 100)
models <- map(cv2$train, ~ lm(mpg ~ wt, data = .))
errs <- map2_dbl(models, cv2$test, rmse)
hist(errs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sim}
\alias{sim}
\alias{sim1}
\alias{sim2}
\alias{sim3}
\alias{sim4}
\title{Simple simulated datasets}
\usage{
sim1

sim2

sim3

sim4
}
\description{
These simple simulated datasets are useful for teaching modelling
basics.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_matrix.R
\name{model_matrix}
\alias{model_matrix}
\title{Construct a design matrix}
\usage{
model_matrix(data, formula, ...)
}
\arguments{
\item{data}{A data frame}

\item{formula}{A modelling formula}

\item{...}{Other arguments passed onto \code{\link[stats:model.matrix]{stats::model.matrix()}}}
}
\value{
A tibble.
}
\description{
This is a thin wrapper around \code{\link[stats:model.matrix]{stats::model.matrix()}} which
returns a tibble. Use it to determine how your modelling formula is
translated into a matrix, an thence into an equation.
}
\examples{
model_matrix(mtcars, mpg ~ cyl)
model_matrix(iris, Sepal.Length ~ Species)
model_matrix(iris, Sepal.Length ~ Species - 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{model-quality}
\alias{model-quality}
\alias{mse}
\alias{rmse}
\alias{mae}
\alias{rsquare}
\alias{qae}
\alias{mape}
\alias{rsae}
\title{Compute model quality for a given dataset}
\usage{
mse(model, data)

rmse(model, data)

mae(model, data)

rsquare(model, data)

qae(model, data, probs = c(0.05, 0.25, 0.5, 0.75, 0.95))

mape(model, data)

rsae(model, data)
}
\arguments{
\item{model}{A model}

\item{data}{The dataset}

\item{probs}{Numeric vector of probabilities}
}
\description{
Three summaries are immediately interpretible on the scale of the response
variable:
\itemize{
\item \code{rmse()} is the root-mean-squared-error
\item \code{mae()} is the mean absolute error
\item \code{qae()} is quantiles of absolute error.
}

Other summaries have varying scales and interpretations:
\itemize{
\item \code{mape()} mean absolute percentage error.
\item \code{rsae()} is the relative sum of absolute errors.
\item \code{mse()} is the mean-squared-error.
\item \code{rsquare()} is the variance of the predictions divided by the
variance of the response.
}
}
\examples{
mod <- lm(mpg ~ wt, data = mtcars)
mse(mod, mtcars)
rmse(mod, mtcars)
rsquare(mod, mtcars)
mae(mod, mtcars)
qae(mod, mtcars)
mape(mod, mtcars)
rsae(mod, mtcars)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictions.R
\name{add_predictions}
\alias{add_predictions}
\alias{spread_predictions}
\alias{gather_predictions}
\title{Add predictions to a data frame}
\usage{
add_predictions(data, model, var = "pred", type = NULL)

spread_predictions(data, ..., type = NULL)

gather_predictions(data, ..., .pred = "pred", .model = "model", type = NULL)
}
\arguments{
\item{data}{A data frame used to generate the predictions.}

\item{model}{\code{add_predictions} takes a single \code{model};}

\item{var}{The name of the output column, default value is \code{pred}}

\item{type}{Prediction type, passed on to \code{stats::predict()}. Consult
\code{predict()} documentation for given \code{model} to determine valid values.}

\item{...}{\code{gather_predictions} and \code{spread_predictions} take
multiple models. The name will be taken from either the argument
name of the name of the model.}

\item{.pred, .model}{The variable names used by \code{gather_predictions}.}
}
\value{
A data frame. \code{add_prediction} adds a single new column,
with default name \code{pred}, to the input \code{data}.
\code{spread_predictions} adds one column for each model. \code{gather_predictions}
adds two columns \code{.model} and \code{.pred}, and repeats the input rows for each
model.
}
\description{
Add predictions to a data frame
}
\examples{
df <- tibble::tibble(
  x = sort(runif(100)),
  y = 5 * x + 0.5 * x ^ 2 + 3 + rnorm(length(x))
)
plot(df)

m1 <- lm(y ~ x, data = df)
grid <- data.frame(x = seq(0, 1, length = 10))
grid \%>\% add_predictions(m1)

m2 <- lm(y ~ poly(x, 2), data = df)
grid \%>\% spread_predictions(m1, m2)
grid \%>\% gather_predictions(m1, m2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formulas.R
\name{formulas}
\alias{formulas}
\alias{formulae}
\title{Create a list of formulas}
\usage{
formulas(.response, ...)

formulae(.response, ...)
}
\arguments{
\item{.response}{A one-sided formula used as the left-hand side of
all resulting formulas.}

\item{...}{List of formulas whose right-hand sides will be merged
to \code{.response}.}
}
\description{
\code{formulas()} creates a list of two-sided formulas by merging a
unique left-hand side to a list of right-hand sides.
}
\examples{
# Provide named arguments to create a named list of formulas:
models <- formulas(~lhs,
  additive = ~var1 + var2,
  interaction = ~var1 * var2
)
models$additive

# The formulas are created sequentially, so that you can refer to
# previously created formulas:
formulas(~lhs,
  linear = ~var1 + var2,
  hierarchical = add_predictors(linear, ~(1 | group))
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permute.R
\name{permute}
\alias{permute}
\alias{permute_}
\title{Generate \code{n} permutation replicates.}
\usage{
permute(data, n, ..., .id = ".id")

permute_(data, n, columns, .id = ".id")
}
\arguments{
\item{data}{A data frame}

\item{n}{Number of permutations to generate.}

\item{...}{Columns to permute. This supports bare column names or dplyr
\link[dplyr:reexports]{dplyr::select_helpers}}

\item{.id}{Name of variable that gives each model a unique integer id.}

\item{columns}{In \code{permute_}, vector of column names to permute.}
}
\value{
A data frame with \code{n} rows and one column: \code{perm}
}
\description{
A permutation test involves permuting one or more variables in a data set
before performing the test, in order to break any existing relationships
and simulate the null hypothesis. One can then compare the true statistic
to the generated distribution of null statistics.
}
\examples{

library(purrr)
perms <- permute(mtcars, 100, mpg)

models <- map(perms$perm, ~ lm(mpg ~ wt, data = .))
glanced <- map_df(models, broom::glance, .id = "id")

# distribution of null permutation statistics
hist(glanced$statistic)
# confirm these are roughly uniform p-values
hist(glanced$p.value)

# test against the unpermuted model to get a permutation p-value
mod <- lm(mpg ~ wt, mtcars)
mean(glanced$statistic > broom::glance(mod)$statistic)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{heights}
\alias{heights}
\title{Height and income data.}
\format{
\describe{
\item{income}{Yearly income. The top two percent of values were averaged
and that average was used to replace all values in the top range. }
\item{height}{Height, in inches}
\item{weight}{Weight, in pounds}
\item{age}{Age, in years, between 47 and 56.}
\item{marital}{Marital status}
\item{sex}{Sex}
\item{education}{Years of education}
\item{afqt}{Percentile score on Armed Forces Qualification Test.}
}
}
\usage{
heights
}
\description{
You might have heard that taller people earn more. Is it true? You can
try and answer the question by exploring this dataset extracted from the
\href{https://www.nlsinfo.org}{National Longitudinal Study}, which is
sponsored by the U.S. Bureau of Labor Statistics.
}
\details{
This contains data as at 2012.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formulas.R
\name{add_predictors}
\alias{add_predictors}
\title{Add predictors to a formula}
\usage{
add_predictors(f, ..., fun = "+")
}
\arguments{
\item{f}{A formula.}

\item{...}{Formulas whose right-hand sides will be merged to
\code{f}.}

\item{fun}{A function name indicating how to merge the right-hand
sides.}
}
\description{
This merges a one- or two-sided formula \code{f} with the
right-hand sides of all formulas supplied in \code{...}.
}
\examples{
f <- lhs ~ rhs
add_predictors(f, ~var1, ~var2)

# Left-hand sides are ignored:
add_predictors(f, lhs1 ~ var1, lhs2 ~ var2)

# fun can also be set to a function like "*":
add_predictors(f, ~var1, ~var2, fun = "*")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permute.R
\name{resample_permutation}
\alias{resample_permutation}
\title{Create a resampled permutation of a data frame}
\usage{
resample_permutation(data, columns, idx = NULL)
}
\arguments{
\item{data}{A data frame}

\item{columns}{Columns to be permuted}

\item{idx}{Indices to permute by. If not given, generates them randomly}
}
\value{
A permutation object; use as.data.frame to convert to a permuted
data frame
}
\description{
Create a resampled permutation of a data frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partition.R
\name{resample_partition}
\alias{resample_partition}
\title{Generate an exclusive partitioning of a data frame}
\usage{
resample_partition(data, p)
}
\arguments{
\item{data}{A data frame}

\item{p}{A named numeric vector giving where the value is the probability
that an observation will be assigned to that group.}
}
\description{
Generate an exclusive partitioning of a data frame
}
\examples{
ex <- resample_partition(mtcars, c(test = 0.3, train = 0.7))
mod <- lm(mpg ~ wt, data = ex$train)
rmse(mod, ex$test)
rmse(mod, ex$train)
}
\seealso{
Other resampling techniques: 
\code{\link{bootstrap}()},
\code{\link{resample_bootstrap}()},
\code{\link{resample}()}
}
\concept{resampling techniques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/na-warn.R
\name{na.warn}
\alias{na.warn}
\title{Handle missing values with a warning}
\usage{
na.warn(object)
}
\arguments{
\item{object}{A data frame}
}
\description{
This NA handler ensures that those models that support the
\code{na.action} parameter do not silently drop missing values. It
wraps around \code{\link[stats:na.fail]{stats::na.exclude()}} so that there is one
prediction/residual for input row. To apply it globally, run
\code{options(na.action = na.warn)}.
}
\examples{
df <- tibble::tibble(
  x = 1:10,
  y = c(5.1, 9.7, NA, 17.4, 21.2, 26.6, 27.9, NA, 36.3, 40.4)
)
# Default behaviour is to silently drop
m1 <- lm(y ~ x, data = df)
resid(m1)

# Use na.action = na.warn to warn
m2 <- lm(y ~ x, data = df, na.action = na.warn)
resid(m2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-grid.R
\name{data_grid}
\alias{data_grid}
\title{Generate a data grid.}
\usage{
data_grid(data, ..., .model = NULL)
}
\arguments{
\item{data}{A data frame}

\item{...}{Variables passed on to \code{\link[tidyr:expand]{tidyr::expand()}}}

\item{.model}{A model.  If supplied, any predictors needed for the model
not present in \code{...} will be filled in with "\link{typical}" values.}
}
\description{
To visualise a model, it is very useful to be able to generate an
evenly spaced grid of points from the data. \code{data_grid} helps you
do this by wrapping around \code{\link[tidyr:expand]{tidyr::expand()}}.
}
\examples{
data_grid(mtcars, vs, am)

# For continuous variables, seq_range is useful
data_grid(mtcars, mpg = mpg)
data_grid(mtcars, mpg = seq_range(mpg, 10))

# If you supply a model, missing predictors will be filled in with
# typical values
mod <- lm(mpg ~ wt + cyl + vs, data = mtcars)
data_grid(mtcars, .model = mod)
data_grid(mtcars, cyl = seq_range(cyl, 9), .model = mod)
}
\seealso{
\code{\link[=seq_range]{seq_range()}} for generating ranges from continuous
variables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residuals.R
\name{add_residuals}
\alias{add_residuals}
\alias{spread_residuals}
\alias{gather_residuals}
\title{Add residuals to a data frame}
\usage{
add_residuals(data, model, var = "resid")

spread_residuals(data, ...)

gather_residuals(data, ..., .resid = "resid", .model = "model")
}
\arguments{
\item{data}{A data frame used to generate the residuals}

\item{model, var}{\code{add_residuals} takes a single \code{model}; the
output column will be called \code{resid}}

\item{...}{\code{gather_residuals} and \code{spread_residuals} take
multiple models. The name will be taken from either the argument
name of the name of the model.}

\item{.resid, .model}{The variable names used by \code{gather_residuals}.}
}
\value{
A data frame. \code{add_residuals} adds a single new column,
\code{.resid}, to the input \code{data}. \code{spread_residuals} adds
one column for each model. \code{gather_predictions} adds two columns
\code{.model} and \code{.resid}, and repeats the input rows for
each model.
}
\description{
Add residuals to a data frame
}
\examples{
df <- tibble::tibble(
  x = sort(runif(100)),
  y = 5 * x + 0.5 * x ^ 2 + 3 + rnorm(length(x))
)
plot(df)

m1 <- lm(y ~ x, data = df)
df \%>\% add_residuals(m1)

m2 <- lm(y ~ poly(x, 2), data = df)
df \%>\% spread_residuals(m1, m2)
df \%>\% gather_residuals(m1, m2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{resample_bootstrap}
\alias{resample_bootstrap}
\title{Generate a boostrap replicate}
\usage{
resample_bootstrap(data)
}
\arguments{
\item{data}{A data frame}
}
\description{
Generate a boostrap replicate
}
\examples{
coef(lm(mpg ~ wt, data = resample_bootstrap(mtcars)))
coef(lm(mpg ~ wt, data = resample_bootstrap(mtcars)))
coef(lm(mpg ~ wt, data = resample_bootstrap(mtcars)))
}
\seealso{
Other resampling techniques: 
\code{\link{bootstrap}()},
\code{\link{resample_partition}()},
\code{\link{resample}()}
}
\concept{resampling techniques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelr.R
\docType{package}
\name{modelr-package}
\alias{modelr}
\alias{modelr-package}
\title{modelr: Modelling Functions that Work with the Pipe}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Functions for modelling that help you seamlessly integrate modelling into a pipeline of data manipulation and visualisation.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://modelr.tidyverse.org}
  \item \url{https://github.com/tidyverse/modelr}
  \item Report bugs at \url{https://github.com/tidyverse/modelr/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@rstudio.com}

Other contributors:
\itemize{
  \item RStudio [copyright holder, funder]
}

}
\keyword{internal}
